//------------------------------------------------------------------------------
// Author:		Gero Mueller <gero.mueller@cloo.de>
// License:		GPL v2 (only)
//------------------------------------------------------------------------------

require( "missile.class" );

class Combat extends Entity
{
	rigidbody = null;
	chassis_collision_mesh = null;
	trafonode = null;
	modelnode = null;
	lifetime = 0;
	target = null;
	lastshift = 1.0;
	//waypoints = [];
	currentWaypoint = 0;
	playercontrol = false;
	missile_reload = 0;
	sound = null;
	
	constructor()
	{
		rigidbody = RigidBody( this );
		
		chassis_collision_mesh = rigidbody.addCollisionMesh( "combat", 200.0 );
		//rigidbody.setCollisionMeshPosition( chassis_collision_mesh, 0.0, 0.0, 0.0 );
		//rigidbody.setCollisionMeshRotation( chassis_collision_mesh, 0.0, 0.0, 1.57 );

		trafonode = TransformationNode( null, this.rigidbody );
		modelnode = ModelNode( trafonode, "combat" );
		//modelnode.setRelativeTranslation( 0.0, 0.0, 5.0 );
	}
	
	function setPosition( p )
	{
		rigidbody.setPosition( p );
	}
	
	function tick( time )
	{
	/*
		lifetime += time;

		if( lifetime > 20.0 )
			return 1;
		
		return 0;
		*/
	}
	
	function handleCollision( entity1, geom1, entity2, geom2 )
	{
		print("Collision");

		if( playercontrol )
		{
			if( sound == null )
				sound = Sound( "crash.ogg" );
			
			sound.play();
		}
	}
	
	function step( time )
	{
		if( playercontrol )
		{
				local r = Vector();
				if( isKeyPressed( 284 ) ) // down
					r.x += 5.0;
				if( isKeyPressed( 283 ) ) // up
					r.x -= 5.0;

				if( isKeyPressed( 285 ) ) // left
					r.y += 5.0;
				if( isKeyPressed( 286 ) ) // right
					r.y -= 5.0;

				if( isKeyPressed( 'Q' ) )
					r.z += 5.0;
				if( isKeyPressed( 'E' ) )
					r.z -= 5.0;

				local q = rigidbody.getRotation();
				r -= q.applyInversed( rigidbody.getAngularVelocity() );	

				rigidbody.applyLocalForce(
					Vector( 0.0, -r.x * 8000.0, 0.0 ), Vector( 0.0, 0.0, 1.0 ) );
				rigidbody.applyLocalForce(
					Vector( 0.0, r.x * 8000.0, 0.0 ), Vector( 0.0, 0.0, -1.0 ) );

				rigidbody.applyLocalForce(
					Vector( r.y * 5000.0, 0.0, 0.0 ), Vector( 0.0, 0.0, 2.0 ) );
				rigidbody.applyLocalForce(
					Vector( -r.y * 5000.0, 0.0, 0.0 ), Vector( 0.0, 0.0, -2.0 ) );

				rigidbody.applyLocalForce(
					Vector( 0.0, r.z * 5000.0, 0.0 ), Vector( 2.0, 0.0, 0.0 ) );
				rigidbody.applyLocalForce(
					Vector( 0.0, -r.z * 5000.0, 0.0 ), Vector( -2.0, 0.0, 0.0 ) );

			
			
			if( isKeyPressed( 'S' ) )
			{
				rigidbody.applyLocalForce(
					Vector( 0.0, 0.0, 100000.0 ), Vector( 0.0, 0.0, 0.0)  );
			}
			
			if( isKeyPressed( 'W' ) )
			{
				rigidbody.applyLocalForce(
					Vector( 0.0, 0.0, -100000.0 ), Vector( 0.0, 0.0, 0.0) );
			}
			if( isKeyPressed( 'A' ) )
			{
				rigidbody.applyLocalForce(
					Vector( -100000.0, 0.0, 0.0 ), Vector( 0.0, 0.0, 0.0) );
			}

			if( isKeyPressed( 'D' ) )
			{
				rigidbody.applyLocalForce(
					Vector( 100000.0, 0.0, 0.0), Vector( 0.0, 0.0, 0.0 ) );
			}

			if( isKeyPressed( 'Q' ) )
			{
				rigidbody.applyLocalForce(
					Vector( -100000.0, 0.0, 0.0), Vector( 0.0, 1.0, 0.0 ) );
				rigidbody.applyLocalForce(
					Vector( 100000.0, 0.0, 0.0), Vector( 0.0, -1.0, 0.0 ) );
			}

			if( isKeyPressed( 'E' ) )
			{
				rigidbody.applyLocalForce(
					Vector( -100000.0, 0.0, 0.0), Vector( 0.0, -1.0, 0.0 ) );
				rigidbody.applyLocalForce(
					Vector( 100000.0, 0.0, 0.0), Vector( 0.0, 1.0, 0.0 ) );
			}

			if( isKeyPressed( 'R' ) )
			{
				rigidbody.applyLocalForce(
					Vector( 0.0, 100000.0, 0.0), Vector( 0.0, 0.0, 0.0 ) );
			}

			if( isKeyPressed( 'F' ) )
			{
				rigidbody.applyLocalForce(
					Vector( 0.0, -100000.0, 0.0), Vector( 0.0, 0.0, 0.0 ) );
			}
	
			if( isKeyPressed( 32 ) )
			{
				if( missile_reload <= 0.0 )
				{
					local missile = Missile();
					local rot = rigidbody.getRotation();
					local pos = rigidbody.getPosition();
					//missile.rigidbody.setPosition(+ rot.apply( Vector( 0.0, -1.0, -25.0 ) ) );
					missile.rigidbody.setPosition( pos + rot.apply( Vector( 0.0, -1.0, -25.0 ) ) );
					missile.rigidbody.setRotation( rot );
					missile.rigidbody.setLinearVelocity( rigidbody.getLinearVelocity() );
					
					local mpos = missile.rigidbody.getPosition();
					local mrot = missile.rigidbody.getRotation();
					
					::EntityList.append( missile );
					missile_reload = 0.3;
				}	
			}

			missile_reload -= time;
	/*	
	
			if( isKeyPressed( 285 ) ) // left
			{
				rigidbody.applyLocalForce(
					Vector( -20000.0, 0.0, 0.0), Vector( 0.0, 0.0, -1.0 ) );
				rigidbody.applyLocalForce(
					Vector( 20000.0, 0.0, 0.0), Vector( 0.0, 0.0, 1.0 ) );
			}
						
			if( isKeyPressed( 286 ) ) // right
			{
				rigidbody.applyLocalForce(
					Vector( 20000.0, 0.0, 0.0), Vector( 0.0, 0.0, -1.0 ) );
				rigidbody.applyLocalForce(
					Vector( -20000.0, 0.0, 0.0), Vector( 0.0, 0.0, 1.0 ) );
			}

			if( isKeyPressed( 283 ) ) // up
			{
				rigidbody.applyLocalForce(
					Vector( 0.0, -20000.0, 0.0), Vector( 0.0, 0.0, -1.0 ) );
				rigidbody.applyLocalForce(
					Vector( 0.0, 20000.0, 0.0), Vector( 0.0, 0.0, 1.0 ) );
			}

			if( isKeyPressed( 284 ) ) // down
			{
				rigidbody.applyLocalForce(
					Vector( 0.0, 20000.0, 0.0), Vector( 0.0, 0.0, -1.0 ) );
				rigidbody.applyLocalForce(
					Vector( 0.0, -20000.0, 0.0), Vector( 0.0, 0.0, 1.0 ) );
			}
*/
		}
/*
		else if( target == null )
		{
			if( lastshift > 1.0 )
			{
				//local r = Vector( (rand() - 0.5), (rand()-0.5), (rand()-0.5) );
				local r = Vector( 0.5, 0.5, 0.5 );
				r *= 500000.0;
				rigidbody.applyLocalForce( r , Vector( 0.0, 0.0, 0.0 ) );
				lastshift = 0.0;
			}
			lastshift += time;

			//local p = this.rigidbody.getPosition();
			//p *= -500.0;
			//rigidbody.applyLocalForce( p , Vector( 0.0, 0.0, 0.0 ) );
		}
		else
		{
			local d = target.rigidbody.getPosition() - this.rigidbody.getPosition();
			//d.normalize();
			print( d.length() );
			d *= 500.0;
			rigidbody.applyGlobalForce( d, this.rigidbody.getPosition() );
		}
*/
	}
}
